var MPS = /** @class */ (function () {
    function MPS() {
        this.lines = [];
        for (var i = 0; i < 256; i++) {
            this.lines.push(new MPSLine((i + 1) % 256));
        }
    }
    MPS.prototype.reset = function () {
        this.lines.forEach(function (l, i) { return l.reset((i + 1) % 256); });
    };
    MPS.prototype.isChanged = function () {
        for (var i = 0; i < 256; i++) {
            var line = this.lines[i];
            if (line.comment.trim() != "")
                return true;
            for (var j = 0; j < 16; j++) {
                if (line.openGates[j])
                    return true;
            }
            if (line.fa != ((i + 1) % 256))
                return true;
        }
        return false;
    };
    MPS.prototype.loadMPS = function (fileContent) {
        this.reset();
        for (var i = 0; i < 256; i++) {
            //let line : Uint8Array = new Uint8Array(fileContent, 16*i, 16);
            var theLine = this.lines[i];
            for (var b = 0; b < 8; b++) {
                // Tore 0 bis 7:
                if ((fileContent[16 * i] & (1 << b)) != 0)
                    theLine.openGates[b] = true;
                // Tore 8 bis F:
                if ((fileContent[16 * i + 1] & (1 << b)) != 0)
                    theLine.openGates[b + 8] = true;
            }
            theLine.fa = fileContent[16 * i + 2];
            var commentLen = fileContent[16 * i + 3];
            if (commentLen > 0 && commentLen <= 12) {
                theLine.comment = new TextDecoder().decode(fileContent.slice(16 * i + 4, 16 * i + commentLen + 4));
            }
        }
    };
    MPS.prototype.loadMPD = function (fileContent) {
        var _this = this;
        this.reset();
        var lines = fileContent.split(/\r?\n/);
        lines.forEach(function (l, i) {
            if (i > 2 && l.trim() != "") // die ersten 3 Zeilen können übersprungen werden
             {
                var lnr = parseInt(l.substring(0, 2), 16);
                if (_this.lines[lnr] === undefined) {
                    console.assert(false);
                }
                _this.lines[lnr].comment = l.substring(3, l.length - 34).trim();
                for (var g = 0; g < 16; g++) {
                    var c = l.substring(l.length - 34 + 2 * g, l.length - 34 + 2 * g + 1);
                    if (c != ".")
                        _this.lines[lnr].openGates[g] = true;
                }
                _this.lines[lnr].fa = parseInt(l.substring(l.length - 2), 16);
            }
        });
    };
    MPS.prototype.saveMPD = function (name) {
        var ergebnis = "; MikrSimD-Programm-Speicher: " + name + "\n;\nAD Befehl                    0 1 2 3 4 5 6 7 8 9 A B C D E F FA";
        for (var i = 0; i < 256; i++) {
            ergebnis += "\n" + decimalToHex(i) + " " + padEnd(this.lines[i].comment, 26, ' ');
            for (var j = 0; j < 16; j++) {
                ergebnis += this.lines[i].openGates[j] ? decimalToHex(j, 1) : ".";
                ergebnis += " ";
            }
            ergebnis += decimalToHex(this.lines[i].fa);
        }
        return ergebnis;
    };
    MPS.prototype.saveMPS = function () {
        var ergebnis = new Uint8Array(4096);
        for (var i = 0; i < 256; i++) {
            var byte1 = 0;
            var byte2 = 0;
            for (var g = 0; g < 8; g++) {
                if (this.lines[i].openGates[g])
                    byte1 |= (1 << g);
                if (this.lines[i].openGates[g + 8])
                    byte2 |= (1 << g);
            }
            ergebnis[16 * i] = byte1;
            ergebnis[16 * i + 1] = byte2;
            ergebnis[16 * i + 2] = this.lines[i].fa;
            ergebnis[16 * i + 3] = this.lines[i].comment.length > 12 ? 12 : this.lines[i].comment.length;
            var comment = new TextEncoder().encode(this.lines[i].comment);
            for (var j = 0; j < comment.length; j++) {
                ergebnis[16 * i + 4 + j] = comment[j];
            }
        }
        return ergebnis;
    };
    MPS.prototype.insertRow = function (idx) {
        for (var i = 255; i > idx; i--) {
            this.lines[i].comment = this.lines[i - 1].comment;
            this.lines[i].fa = this.lines[i - 1].fa;
            for (var g = 0; g < 16; g++) {
                this.lines[i].openGates[g] = this.lines[i - 1].openGates[g];
            }
        }
        for (var g = 0; g < 16; g++) {
            this.lines[idx].openGates[g] = false;
        }
        for (var i = 0; i < 256; i++) {
            if (this.lines[i].fa > idx && this.lines[i].fa != 0) {
                this.lines[i].fa = (this.lines[i].fa + 1) % 256;
            }
        }
        this.lines[idx].comment = "";
        this.lines[idx].fa = idx + 1;
    };
    MPS.prototype.removeRow = function (idx) {
        for (var i = idx; i < 255; i++) {
            this.lines[i].comment = this.lines[i + 1].comment;
            this.lines[i].fa = this.lines[i + 1].fa;
            for (var g = 0; g < 16; g++) {
                this.lines[i].openGates[g] = this.lines[i + 1].openGates[g];
            }
        }
        this.lines[255].comment = "";
        this.lines[255].fa = 0;
        for (var g = 0; g < 16; g++) {
            this.lines[255].openGates[g] = false;
        }
        for (var i = 0; i < 256; i++) {
            if (this.lines[i].fa != 0 && this.lines[i].fa > idx) {
                this.lines[i].fa--;
            }
        }
    };
    return MPS;
}());
var MPSLine = /** @class */ (function () {
    function MPSLine(fa) {
        this.fa = fa;
        this.comment = "";
        this.openGates = new Array(16);
        this.reset();
    }
    MPSLine.prototype.reset = function (fa) {
        for (var i = 0; i < 16; i++) {
            this.openGates[i] = false;
        }
        this.comment = "";
        if (fa !== undefined)
            this.fa = fa;
    };
    MPSLine.prototype.getErrorMessage = function (n) {
        if (n == 11 || n == 12 || n == 15) // RAM-Zugriff oder MPS: Es darf nix anderes offen sein
         {
            for (var i = 0; i < 16; i++) {
                if (this.openGates[i]) {
                    return "Tor " + decimalToHex(n, 1) + " kann nicht geöffnet werden, es ist ein anderes Tor geöffnet.";
                }
            }
        }
        else {
            if (this.openGates[11] || this.openGates[12])
                return "Tor " + decimalToHex(n, 1) + " kann nicht geöffnet werden, es erfolgt ein Zugriff aufs RAM.";
            if (this.openGates[15])
                return "Tor " + decimalToHex(n, 1) + " kann nicht geöffnet werden, denn Tor F ist offen.";
            if (n == 1 || n == 3 || n == 5) // linker Eingang
             {
                if (this.openGates[1] || this.openGates[3] || this.openGates[5]) // Buskonflikt links
                    return "Tor " + decimalToHex(n, 1) + " kann nicht geöffnet werden, das führt zu einem Buskonflikt.";
            }
            if (n == 7 || n == 9) {
                if (this.openGates[7] || this.openGates[9]) // Buskonflikt rechts
                    return "Tor " + decimalToHex(n, 1) + " kann nicht geöffnet werden, das führt zu einem Buskonflikt.";
            }
        }
        return "";
    };
    MPSLine.prototype.toggleGate = function (n, withCallback) {
        if (this.openGates[n]) // Schließen
         {
            this.openGates[n] = false;
        }
        else {
            var error = this.getErrorMessage(n);
            if (error != "")
                throw new Error(error);
            this.openGates[n] = true;
        }
        if (withCallback === undefined || withCallback === true) {
            saveToSystemStorage();
        }
    };
    MPSLine.prototype.tokenize = function (input) {
        var tokens = input.match(/\b[a-f0-9]+\b|\b[a-z]+\b|[,\[\]]/g);
        return tokens ? tokens : [];
    };
    MPSLine.prototype.matches = function (assemblerCommand, variableMap) {
        if (this.comment.trim() == "")
            return undefined;
        assemblerCommand = assemblerCommand.trim();
        if (assemblerCommand == "")
            return undefined;
        var assemblerParts = this.tokenize(assemblerCommand);
        var commentParts = this.tokenize(this.comment.toLowerCase());
        if (assemblerParts.length != commentParts.length) {
            var idx = commentParts.indexOf("[");
            if (idx >= 0 && idx <= commentParts.length - 3 && (commentParts[idx + 1] == "n" || commentParts[idx + 1] == "nn") && commentParts[idx + 2] == "]") {
                commentParts[idx] += commentParts[idx + 1] + commentParts[idx + 2];
                commentParts.splice(idx + 1, 2);
            }
        }
        if (assemblerParts.length != commentParts.length)
            return undefined;
        var parameter = undefined;
        var hasParameter = false;
        for (var i = 0; i < commentParts.length; i++) {
            var ac = assemblerParts[i];
            var cp = commentParts[i];
            if (cp == "nn" || cp == "n") {
                hasParameter = true;
                if (parameter !== undefined) // sollte nicht passieren, dass zweimal "nn" oder "n" drinsteht
                    return undefined;
                if (ac.match(/^[a-f0-9]+$/i) === null)
                    return undefined;
                parameter = parseInt(ac, 16);
                if (isNaN(parameter)) // Parameter ist keine richtige Zahl -> geht nicht
                    return undefined;
            }
            else if (cp == "[nn]" || cp == "[n]") {
                hasParameter = true;
                if (parameter !== undefined) // sollte nicht passieren, dass zweimal "nn" oder "n" drinsteht
                    return undefined;
                if (ac.charAt(0) == "[" && ac.charAt(ac.length - 1) == "]") // muss mit [ beginnen und mit ] enden
                 {
                    if (ac.substring(1, ac.length - 1).match(/^[a-f0-9]+$/i) === null)
                        return undefined;
                    parameter = parseInt(ac.substring(1, ac.length - 1), 16);
                    if (isNaN(parameter)) // Parameter ist keine richtige Zahl -> geht nicht
                        return undefined;
                }
                else {
                    var idx = variableMap.indexOf(ac);
                    if (idx < 0)
                        return undefined;
                    parameter = idx;
                }
            }
            else if (ac != cp) {
                return undefined;
            }
        }
        if (hasParameter) // Befehl sollte einen Parameter haben
         {
            return parameter;
        }
        if (parameter === undefined) // Befehl sollte keinen Parameter haben, es wurde aber einer gefunden -> kein Match
            return 0;
        else
            return undefined;
    };
    return MPSLine;
}());
var ObservableInt = /** @class */ (function () {
    function ObservableInt(v) {
        this.value = v;
        this.callbacks = [];
    }
    ObservableInt.prototype.addObserver = function (f) {
        this.callbacks.push(f);
    };
    ObservableInt.prototype.get = function () {
        return this.value;
    };
    ObservableInt.prototype.set = function (v, withCallbacks) {
        var old = this.value;
        this.value = v;
        if (withCallbacks === undefined || withCallbacks === true) {
            for (var i = 0; i < this.callbacks.length; i++) {
                this.callbacks[i](old, v);
            }
            saveToSystemStorage();
        }
    };
    return ObservableInt;
}());
var SystemState = /** @class */ (function () {
    function SystemState() {
        this.mps = new MPS();
        this.ram = new Array(256);
        this.comments = new Array(256);
        for (var i = 0; i < 256; i++) {
            this.ram[i] = new ObservableInt(0);
            this.comments[i] = "";
        }
        this.manualGates = new MPSLine(undefined);
        this.ax = new ObservableInt(0);
        this.bx = new ObservableInt(0);
        this.ip = new ObservableInt(0);
        this.op = new ObservableInt(0);
        this.alu = new ObservableInt(0);
        this.dr = new ObservableInt(0);
        this.ar = new ObservableInt(0);
        this.currentMPS = new ObservableInt(0);
        this.manualMode = false;
        this.variableMap = [];
    }
    SystemState.prototype.isRAMChanged = function () {
        if (this.variableMap.filter(function (x) { return x; }).length != 0)
            return true;
        for (var i = 0; i < 256; i++) {
            if (this.ram[i].get() != 0)
                return true;
        }
        return false;
    };
    SystemState.prototype.removeRAMRow = function (idx) {
        if (idx % 2 != 0)
            idx--;
        for (var i = idx; i < 256 - 2; i += 2) {
            this.ram[i].set(this.ram[i + 2].get());
            this.ram[i + 1].set(this.ram[i + 3].get());
            this.comments[i] = this.comments[i + 2];
            this.comments[i + 1] = this.comments[i + 3];
        }
        this.ram[254].set(0);
        this.ram[255].set(0);
        for (var i = 0; i < 256; i += 2) {
            if (this.refersToAddress(this.ram[i].get()) && this.ram[i + 1].get() >= idx + 2) // wenn sich der Befehl auf eine Speicheradresse bezieht, die weiter hinten steht
             {
                this.ram[i + 1].set((this.ram[i + 1].get() + 254) % 256); // +254 entspricht -2, verhält sich aber mit Modulo korrekt
            }
        }
        var newVarMap = [];
        this.variableMap.forEach(function (name, i) {
            if (i >= idx + 2) {
                newVarMap[i - 2] = name;
            }
            else {
                newVarMap[i] = name;
            }
        });
        this.variableMap = newVarMap;
    };
    SystemState.prototype.insertRAMRow = function (idx) {
        if (idx % 2 != 0)
            idx--;
        for (var i = 256 - 2; i > idx; i -= 2) {
            this.ram[i].set(this.ram[i - 2].get());
            this.ram[i + 1].set(this.ram[i - 1].get());
            this.comments[i] = this.comments[i - 2];
            this.comments[i + 1] = this.comments[i - 1];
        }
        this.ram[idx].set(0);
        var tmp = idx + 1;
        this.ram[idx + 1].set(0);
        this.comments[idx] = "";
        this.comments[idx + 1] = "";
        for (var i = 0; i < 256; i += 2) {
            if (this.refersToAddress(this.ram[i].get()) && this.ram[i + 1].get() >= idx) // wenn sich der Befehl auf eine Speicheradresse bezieht, die weiter hinten steht
             {
                this.ram[i + 1].set((this.ram[i + 1].get() + 2) % 256);
            }
        }
        var newVarMap = [];
        this.variableMap.forEach(function (name, i) {
            if (i >= idx) {
                if (i > 253)
                    newVarMap[255] = name;
                else
                    newVarMap[i + 2] = name;
            }
            else {
                newVarMap[i] = name;
            }
        });
        this.variableMap = newVarMap;
    };
    SystemState.prototype.refersToAddress = function (opcode) {
        var op = this.mps.lines[opcode].comment;
        return opcode != 0 && op.indexOf("[") >= 0 || op.indexOf("J") >= 0;
    };
    SystemState.prototype.isManualMode = function () {
        return this.manualMode;
    };
    SystemState.prototype.setManualMode = function (mode) {
        if (!mode) {
            this.manualGates.reset();
            this.currentMPS.set(0);
        }
        this.manualMode = mode;
    };
    SystemState.prototype.getOpenGates = function () {
        var currentGates = this.manualMode ? this.manualGates : this.mps.lines[this.currentMPS.get()];
        return currentGates.openGates.map(function (v, i) { return v ? i : -1; }).filter(function (i) { return i >= 0; });
    };
    SystemState.prototype.getLitLines = function (toALU) {
        var currentGates = this.manualMode ? this.manualGates : this.mps.lines[this.currentMPS.get()];
        var result = {};
        //console.log("Zeile: " + this.currentMPS.get() + "; To ALU: " + toALU);
        //currentGates.openGates.forEach((val, i) => {if (val) console.log(i);});
        if (toALU) {
            if (currentGates.openGates[11]) {
                result["dr_ram"] = true;
                result["ar_ram"] = true;
            }
            if (currentGates.openGates[15]) {
                result["op_mps"] = true;
            }
            if (currentGates.openGates[14]) {
                result["sf_mps"] = true;
            }
            if (currentGates.openGates[1])
                result["ax_alu"] = true;
            else if (currentGates.openGates[3])
                result["bx_alu"] = true;
            else if (currentGates.openGates[5])
                result["ip_alu"] = true;
            if (currentGates.openGates[7])
                result["1_alu"] = true;
            else if (currentGates.openGates[9])
                result["dr_alu"] = true;
            if (currentGates.openGates[13])
                result["minus"] = true;
        }
        else {
            if (currentGates.openGates[12]) {
                result["ram_dr"] = true;
                result["ar_ram"] = true;
            }
            else {
                if (currentGates.openGates[0]) {
                    result["alu_ax"] = true;
                }
                if (currentGates.openGates[2]) {
                    result["alu_bx"] = true;
                }
                if (currentGates.openGates[4]) {
                    result["alu_ip"] = true;
                }
                if (currentGates.openGates[6]) {
                    result["alu_op"] = true;
                }
                if (currentGates.openGates[8]) {
                    result["alu_ar"] = true;
                }
                if (currentGates.openGates[10]) {
                    result["alu_dr"] = true;
                }
            }
        }
        return result;
    };
    SystemState.prototype.updateMPS = function () {
        this.currentMPS.set(this.nextFA);
    };
    SystemState.prototype.busTakt = function (toALU) {
        var currentGates = this.manualMode ? this.manualGates : this.mps.lines[this.currentMPS.get()];
        if (toALU) {
            if (currentGates.openGates[11])
                this.ram[this.ar.get()].set(this.dr.get());
            else {
                var result = 0;
                if (currentGates.openGates[1])
                    result = this.ax.get();
                else if (currentGates.openGates[3])
                    result = this.bx.get();
                else if (currentGates.openGates[5])
                    result = this.ip.get();
                var operatorRight = 0;
                if (currentGates.openGates[7])
                    operatorRight = 1;
                else if (currentGates.openGates[9])
                    operatorRight = this.dr.get();
                if (currentGates.openGates[13]) {
                    result -= operatorRight;
                    while (result < 0) {
                        result += 256;
                    }
                }
                else {
                    result += operatorRight;
                    while (result >= 256) {
                        result -= 256;
                    }
                }
                this.alu.set(result);
            }
        }
        else {
            if (!this.manualMode) {
                this.nextFA = currentGates.fa;
                if (currentGates.openGates[14] && this.ax.get() >= 128) // Sign Flag
                    this.nextFA++;
                if (currentGates.openGates[15])
                    this.nextFA += this.op.get();
                this.nextFA %= 256;
            }
            if (currentGates.openGates[12])
                this.dr.set(this.ram[this.ar.get()].get());
            else {
                if (currentGates.openGates[0])
                    this.ax.set(this.alu.get());
                if (currentGates.openGates[2])
                    this.bx.set(this.alu.get());
                if (currentGates.openGates[4])
                    this.ip.set(this.alu.get());
                if (currentGates.openGates[6])
                    this.op.set(this.alu.get());
                if (currentGates.openGates[8])
                    this.ar.set(this.alu.get());
                if (currentGates.openGates[10])
                    this.dr.set(this.alu.get());
            }
            this.alu.set(0);
        }
    };
    SystemState.prototype.reset = function () {
        this.ax.set(0);
        this.bx.set(0);
        this.ar.set(0);
        this.dr.set(0);
        this.ip.set(0);
        this.op.set(0);
        this.currentMPS.set(0);
        if (this.manualMode) {
            this.manualGates.reset();
        }
    };
    SystemState.prototype.updateMPSLine = function () {
        if (!this.manualMode) {
            this.currentMPS.set(this.nextFA);
            this.nextFA = undefined;
        }
    };
    SystemState.prototype.getMPS = function () {
        return this.mps;
    };
    SystemState.prototype.isHaltCommand = function () {
        var currentLine = this.mps.lines[this.currentMPS.get()];
        return this.currentMPS.get() == currentLine.fa;
    };
    SystemState.prototype.resetRAM = function () {
        for (var i = 0; i < 256; i++) {
            this.ram[i].set(0);
        }
        this.variableMap = [];
    };
    SystemState.prototype.loadRAMRAS = function (fileContent) {
        this.resetRAM();
        var lines = fileContent.split(/\r?\n/);
        var part = 0;
        for (var i = 0; i < lines.length; i++) {
            var line = lines[i];
            if (line == "")
                continue;
            if (line.substring(0, 1) == ";")
                continue;
            if (line.substring(0, 3) == "RAS") {
                part = 1;
            }
            else if (line.substring(0, 3) == "VAR") {
                part = 2;
            }
            else if (part == 1) {
                var idx = parseInt(line.substring(0, 2), 16);
                var cmd = parseInt(line.substring(4, 6), 16);
                if (isNaN(cmd))
                    cmd = 0;
                var par = parseInt(line.substring(7, 9), 16);
                if (isNaN(par))
                    par = 0;
                this.ram[idx].set(cmd);
                this.ram[idx + 1].set(par);
                this.comments[idx] = line.substring(10);
            }
            else if (part == 2) {
                this.parseVariable(line);
            }
        }
    };
    SystemState.prototype.parseVariable = function (line) {
        var idx = parseInt(line.substring(0, 2), 16);
        var v = line.substring(3);
        if (this.variableMap.indexOf(v) != -1)
            throw new Error("Fehler: Doppelter Variablenname '" + v + "'.");
        if (this.variableMap[idx] !== undefined)
            throw new Error("Fehler: Doppelte Variablenadresse '" + line.substring(0, 2) + "'.");
        this.variableMap[idx] = v;
    };
    SystemState.prototype.loadRAMRAD = function (fileContent) {
        this.resetRAM();
        var lines = fileContent.split(/\r?\n/);
        var factor = undefined;
        var variables = false;
        for (var i = 0; i < lines.length; i++) {
            var line = lines[i];
            if (line.substring(0, 1) == ";" || line == "") // Kommentar oder leer -> überspringen
                continue;
            if (line.substring(0, 9) == "RAD 00 01") // zeilenweise Darstellung
             {
                factor = 1;
                continue;
            }
            else if (line.substring(0, 9) == "RAD 00 10") // spaltenweise Darstellung
             {
                factor = 16;
                continue;
            }
            else if (factor === undefined) {
                throw new Error("Fehler: Ungültiges Dateiformat");
            }
            if (variables) {
                this.parseVariable(line);
            }
            else {
                if (line.substring(0, 3) == "VAR") {
                    variables = true;
                    continue;
                }
                if (line.substring(0, 3) == "RAS") // ignorieren
                    continue;
                var firstNumber = parseInt(line.substring(0, 2), 16);
                for (var j = 0; j < 16; j++) {
                    var val = parseInt(line.substring(4 + 3 * j, 4 + 3 * j + 2), 16);
                    if (isNaN(factor * j + firstNumber))
                        console.log(factor * j + firstNumber);
                    this.ram[factor * j + firstNumber].set(val);
                }
            }
        }
    };
    SystemState.prototype.loadRAMBinary = function (fileContent) {
        this.resetRAM();
        for (var i = 0; i < fileContent.length; i++) {
            this.ram[i].set(fileContent[i]);
        }
    };
    SystemState.prototype.getRAMAsString = function (dateiname, format) {
        var ergebnis = "";
        if (format == 'd') // Datenformat
         {
            ergebnis = "; MikrSimD-RAM-Daten: " + dateiname + "\n;\nRAD 00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F\n";
            for (var zeile = 0; zeile < 16; zeile++) {
                ergebnis += decimalToHex(zeile * 16) + " ";
                for (var spalte = 0; spalte < 16; spalte++) {
                    ergebnis += " " + decimalToHex(this.ram[zeile * 16 + spalte].get());
                }
                ergebnis += "\n";
            }
        }
        else // Assemblerformat
         {
            ergebnis = "; MikrSimD-RAM-Assembler: " + dateiname + "\n;\nRAS OP nn Befehl\n";
            for (var i = 0; i < 256; i += 2) {
                ergebnis += decimalToHex(i) + "  " + decimalToHex(this.ram[i].get()) + " " + decimalToHex(this.ram[i + 1].get()) + " " + this.comments[i] + "\n";
            }
        }
        if (this.variableMap.filter(function (x) { return x !== undefined; }).length > 0) {
            ergebnis += "\n\n; Variablendefinitionen\nVAR\n";
            this.variableMap.forEach(function (v, idx) {
                ergebnis += decimalToHex(idx) + " " + v + "\n";
            });
        }
        return ergebnis;
    };
    SystemState.prototype.getRAMBinary = function (length) {
        var ergebnis = new Uint8Array(length);
        for (var i = 0; i < length; i++) {
            ergebnis[i] = this.ram[i].get();
        }
        return ergebnis;
    };
    SystemState.prototype.loadFromStorage = function (storedObject) {
        for (var i = 0; i < 256; i++) {
            var x = storedObject["ram"][i]["value"];
            this.ram[i].set(x, false);
            console.log("RAM", i, "=", x);
            this.mps.lines[i].comment = storedObject["mps"]["lines"][i].comment;
            console.log("MPS", i, "Comment", this.mps.lines[i].comment);
            for (var j = 0; j < 16; j++) {
                this.mps.lines[i].openGates[j] = storedObject["mps"]["lines"][i]["openGates"][j];
                console.log("MPS", i, j, "=", this.mps.lines[i].openGates[j]);
            }
        }
    };
    return SystemState;
}());
//# sourceMappingURL=classes.js.map