function styleLoadButton(id, onloadcallback, typeCallback) {
    var input = $("#" + id);
    var tabidx = input.attr("tabindex");
    var newLabel = $("<label tabindex='" + tabidx + "' class='styled' for='" + id + "'></label>").insertAfter("#" + id).html(input.attr("data-label"));
    if (tabidx !== undefined) {
        newLabel.attr("tabindex", tabidx);
        input.removeAttr("tabindex");
    }
    if (onloadcallback !== undefined) {
        input.on('change', function (evt) {
            var target = evt.target;
            var files = target.files;
            if (files.length == 0)
                return;
            var file = files[0];
            var reader = new FileReader();
            var isText = typeCallback === undefined || typeCallback(file.name);
            reader.onload = function (event) {
                onloadcallback(event.target.result, file.name, isText);
            };
            if (isText)
                reader.readAsText(file);
            else
                reader.readAsArrayBuffer(file);
            input.val(null);
        });
    }
}
function prettyConfirm(question, dimensionen, callback) {
    $("<div title=\"Frage\"><p id=\"pFrage\">" + question + "</p></div>").appendTo($("body")).dialog({
        autoOpen: false,
        height: dimensionen.hoehe,
        maxHeight: dimensionen.hoehe,
        minHeight: dimensionen.hoehe,
        width: dimensionen.breite,
        minWidth: dimensionen.breite,
        modal: true,
        buttons: {
            "OK": function () {
                if (callback !== undefined)
                    callback();
                $(this).dialog("close").remove();
            },
            "Abbrechen": function () {
                $(this).dialog("close").remove();
            }
        }
    }).dialog("open");
}
function showToast(message) {
    $('#snackbar').html(message).addClass("show");
    setTimeout(function () { $('#snackbar').removeClass("show"); }, 3000);
}
function decimalToHex(d, padding) {
    var hex = Number(d).toString(16).toUpperCase();
    padding = padding === undefined || padding === null ? 2 : padding;
    while (hex.length < padding) {
        hex = "0" + hex;
    }
    return hex;
}
function makeTDEditable(td, saveCallback, validateCallback, selectNextTD) {
    td.on("click", function (event) {
        if ($(this).children("input").length > 0)
            return false;
        var tdObj = $(this);
        var preText = tdObj.html();
        var inputObj = $("<input type='text' />");
        tdObj.html("");
        inputObj
            .width(tdObj.width() - 3)
            .height(tdObj.height() - 1)
            .css({ border: "0px", fontSize: "15px" })
            .val(preText)
            .appendTo(tdObj)
            .attr("maxlength", tdObj.attr("maxlength"))
            .trigger("focus")
            .trigger("select");
        var tabEnterHandler = function () {
            var newText = inputObj.val();
            writeBack(inputObj, tdObj);
            if (validateCallback !== undefined && !validateCallback(newText)) {
                showToast("Ungültige Eingabe '" + newText + "'");
                tdObj.html(preText);
            }
            else {
                saveCallback(newText);
            }
        };
        if (selectNextTD !== undefined) {
            inputObj.on("keydown", function (event) {
                if (event.key == "Tab") {
                    event.preventDefault();
                    tabEnterHandler();
                    selectNextTD();
                }
            });
        }
        inputObj.on("keyup", function (event) {
            if ("Enter" == event.key) { // press ENTER-key
                tabEnterHandler();
            }
            else if ("Escape" == event.key) { // press ESC-key
                tdObj.html(preText);
            }
        });
        inputObj.on("blur", function () {
            var newText = inputObj.val();
            writeBack(inputObj, tdObj);
            if (validateCallback !== undefined && !validateCallback(newText)) {
                showToast("Ungültige Eingabe '" + newText + "'");
                tdObj.html(preText);
            }
            else {
                saveCallback(newText);
            }
        });
        inputObj.on("click", function () {
            return false;
        });
    });
}
function writeBack(input, td) {
    var text = $(input).val();
    td.html(text);
    return text;
}
function endsWith(s1, s2) {
    var l1 = s1.length;
    var l2 = s2.length;
    if (l1 < l2)
        return false;
    return s1.substring(l1 - l2) == s2;
}
function padEnd(s, len, fillChar) {
    while (s.length < len) {
        s += fillChar;
    }
    return s;
}
function bindContextMenu(element, menuID, setParametersCallback) {
    $(element).on("contextmenu", function (e) {
        e.preventDefault();
        var evt = e;
        setPosition(menuID, evt.pageY, evt.pageX);
        setParametersCallback();
        return false;
    });
}
function toggleMenu(id, command) {
    var _this = this;
    if (command == 'block') {
        // alle anderen ausblenden
        $(".contextmenu:not(" + id + ")").fadeOut(200, function () { return $(_this).css('display', "none"); });
        $(id).fadeIn(200, function () {
            $(this).css('display', command);
        });
    }
    else {
        $(id).fadeOut(200, function () {
            $(this).css('display', command);
        });
    }
}
function setPosition(id, top, left) {
    var w = $(id).width();
    if (left + w + 10 > $(window).width())
        left = $(window).width() - (w + 10);
    $(id).css('left', "".concat(left, "px")).css('top', "".concat(top, "px"));
    toggleMenu(id, 'block');
}
//# sourceMappingURL=utils.js.map